package nl.wolfc.embedded.security;

import java.net.URL;
import java.util.Properties;

import javax.ejb.EJBAccessException;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import junit.framework.TestCase;
import nl.wolfc.security.ejb3.CalculatorLocal;
import nl.wolfc.security.ejb3.HelloWorldLocal;

import org.jboss.ejb3.embedded.EJB3StandaloneBootstrap;
import org.jboss.ejb3.embedded.EJB3StandaloneDeployer;
import org.jboss.security.SecurityAssociation;
import org.jboss.security.SimplePrincipal;

public class SecurityManagerTest extends TestCase {
	private static EJB3StandaloneDeployer deployer;
	
	public static void main(String[] args) {
		junit.textui.TestRunner.run(SecurityManagerTest.class);
	}

	private static EJB3StandaloneDeployer getDeployer() throws Exception {
		if(deployer != null)
			return deployer;
		
		Properties props = System.getProperties();
		props.put("java.naming.factory.initial", "org.jnp.interfaces.LocalOnlyContextFactory");
		props.put("java.naming.factory.url.pkgs", "org.jboss.naming:org.jnp.interfaces");
		
		EJB3StandaloneBootstrap.boot(null);
		EJB3StandaloneBootstrap.deployXmlResource("security-deployment.xml");
		
		deployer = new EJB3StandaloneDeployer();
		URL res = Thread.currentThread().getContextClassLoader().getResource("dummy.txt");
		URL deployDir = EJB3StandaloneDeployer.getContainingUrlFromResource(res, "dummy.txt");
		System.out.println("deployDir = " + deployDir);
		deployer.getDeployDirs().add(deployDir);
		
		// need to set the InitialContext properties that deployer will use
		// to initial EJB containers
		deployer.create();
		deployer.start();
		return deployer;
	}
	
	public SecurityManagerTest() throws Exception {
		getDeployer();
	}
	
	public void test1() throws Exception {
//		//System.out.println(System.getProperty("user.dir"));
//		
//		Properties props = System.getProperties();
//		props.put("java.naming.factory.initial", "org.jnp.interfaces.LocalOnlyContextFactory");
//		props.put("java.naming.factory.url.pkgs", "org.jboss.naming:org.jnp.interfaces");
//		
//		EJB3StandaloneBootstrap.boot(null);
//		EJB3StandaloneBootstrap.deployXmlResource("security-deployment.xml");
//		
//		EJB3StandaloneDeployer deployer = new EJB3StandaloneDeployer();
//		URL res = Thread.currentThread().getContextClassLoader().getResource("dummy.txt");
//		URL deployDir = EJB3StandaloneDeployer.getContainingUrlFromResource(res, "dummy.txt");
//		System.out.println("deployDir = " + deployDir);
//		deployer.getDeployDirs().add(deployDir);
//		
//		// need to set the InitialContext properties that deployer will use
//		// to initial EJB containers
//		deployer.create();
//		deployer.start();
//		
//		Properties userProps = new Properties(System.getProperties());
//		userProps.put(InitialContext.SECURITY_PRINCIPAL, "test");
//		userProps.put(InitialContext.SECURITY_CREDENTIALS, "test");
		
		SecurityAssociation.setPrincipal(new SimplePrincipal("test"));
		SecurityAssociation.setCredential("test".toCharArray());
		
		InitialContext ctx = new InitialContext();
		HelloWorldLocal helloWorld = (HelloWorldLocal) ctx.lookup(HelloWorldLocal.class.getName());
		String msg = helloWorld.sayHelloTo("Tester");
		
		assertEquals("Hello Tester", msg);
		
//		deployer.stop();
//		deployer.destroy();
//		
//		EJB3StandaloneBootstrap.shutdown();
	}
	
	public void testUnknownUser() throws NamingException {
		SecurityAssociation.setPrincipal(new SimplePrincipal("unknown"));
		SecurityAssociation.setCredential("test".toCharArray());
		
		InitialContext ctx = new InitialContext();
		HelloWorldLocal helloWorld = (HelloWorldLocal) ctx.lookup(HelloWorldLocal.class.getName());
		
		try {
			String msg = helloWorld.sayHelloTo("Tester");
			
			assertEquals("Hello Tester", msg);
		}
		catch(EJBAccessException e) {
			return;
		}
		
		fail("expected an EJBAccessException");
	}

	public void testWrongPassword() throws NamingException {
		SecurityAssociation.setPrincipal(new SimplePrincipal("test"));
		SecurityAssociation.setCredential("wrong".toCharArray());
		
		InitialContext ctx = new InitialContext();
		HelloWorldLocal helloWorld = (HelloWorldLocal) ctx.lookup(HelloWorldLocal.class.getName());
		
		try {
			String msg = helloWorld.sayHelloTo("Tester");
			
			assertEquals("Hello Tester", msg);
		}
		catch(EJBAccessException e) {
			return;
		}
		
		fail("expected an EJBAccessException");
	}

	public void testSayBye() throws NamingException {
		SecurityAssociation.setPrincipal(new SimplePrincipal("test"));
		SecurityAssociation.setCredential("wrong".toCharArray());
		
		InitialContext ctx = new InitialContext();
		HelloWorldLocal helloWorld = (HelloWorldLocal) ctx.lookup(HelloWorldLocal.class.getName());
		
		try {
			String msg = helloWorld.sayByeTo("Tester");
			
			assertEquals("Bye Tester", msg);
		}
		catch(EJBAccessException e) {
			return;
		}
		
		fail("expected an EJBAccessException");
	}
	
	public void testOtherDomain() throws Exception {
		SecurityAssociation.setPrincipal(new SimplePrincipal("other"));
		SecurityAssociation.setCredential("other".toCharArray());
		
		InitialContext ctx = new InitialContext();
		CalculatorLocal calculator = (CalculatorLocal) ctx.lookup(CalculatorLocal.class.getName());
		int result = calculator.add(1, 2);
		
		assertEquals(3, result);
	}
}
